#!/usr/bin/env R

library(tidyverse)

# AMR prediction
AMR <- read.delim(file = snakemake@input[[1]], header=FALSE, comment.char="#")
AMR <- AMR %>% select(1,4,5)
colnames(AMR) <- c("ARG","GeneID","Category")

AMR_translation<-read.delim(file = snakemake@input[[2]], header=FALSE)
colnames(AMR_translation) <- c("GeneID", "Gene_Name")
AMR_translation$Gene_Name<- sub('>', '', AMR_translation$Gene_Name)

# Plasmid Prediction
Plasmid <- read.delim(file = snakemake@input[[3]], header=TRUE)
Plasmid <- separate(data= Plasmid, col=label, into=c("left", "right"), sep="\\.")
Plasmid <- Plasmid %>% select(1,2)
colnames(Plasmid) <-c("ContigID","Plasmid_prediction")
Plasmid <- Plasmid %>% filter(ContigID != "contig_name")
Plasmid$ContigID <- sprintf("%010d", Plasmid$ContigID)

Plasmid_translation <- read.delim(file = snakemake@input[[4]], header=FALSE)
colnames(Plasmid_translation) <- c("ContigID", "Contig_Name")
Plasmid_translation$Contig_Name <- sub('>', '', Plasmid_translation$Contig_Name)

# Combine AMR and Plasmid prediction
Contig_gene_list <- read.delim(file = snakemake@input[[5]], header=FALSE)
colnames(Contig_gene_list) <- c("Contig_Name", "Gene_Name")
Contig_gene_list <- merge(Contig_gene_list, AMR_translation, by="Gene_Name", all=TRUE)
Contig_gene_list <- merge(Contig_gene_list, Plasmid_translation, by="Contig_Name", all=TRUE)
Contig_gene_list$ContigID <- as.numeric(as.character(Contig_gene_list$ContigID))
Contig_gene_list$ContigID <- sprintf("%010d", Contig_gene_list$ContigID)

AMR_Plasmid <- merge(Contig_gene_list, AMR, by="GeneID", all = TRUE)
AMR_Plasmid <- merge(AMR_Plasmid, Plasmid, by="ContigID", all = TRUE)
AMR_Plasmid$ARG <- fct_explicit_na(AMR_Plasmid$ARG, na_level = "-")
AMR_Plasmid$Category <- fct_explicit_na(AMR_Plasmid$Category, na_level = "-")
AMR_Plasmid$Plasmid_prediction <- fct_explicit_na(AMR_Plasmid$Plasmid_prediction, na_level = "unclassified")

# Phage prediction
## VirFinder
VirFinder <- read.delim(file = snakemake@input[[6]])
VirFinder <- VirFinder %>% select(1,3,4)
colnames(VirFinder)<- c("ContigID","VirFinder_score","VirFinder_pvalue" )
VirFinder <- VirFinder %>% filter(ContigID != "name")
VirFinder$VirFinder_pvalue <- as.numeric(as.character(VirFinder$VirFinder_pvalue))
VirFinder$VirFinder_score <- as.numeric(as.character(VirFinder$VirFinder_score))
VirFinder$VirFinder_prediction <- ifelse(VirFinder$VirFinder_pvalue <=0.05 & VirFinder$VirFinder_score >=0.70, "Phage", "-")
VirFinder$ContigID <- sprintf("%010d", VirFinder$ContigID)

## VirSorter
vs.pred <- read.csv(file = snakemake@input[[7]], quote="", header=FALSE)
vs.head <- setNames(data.frame(matrix(ncol=12, nrow=0)), c("Contig_id","Nb.genes.contigs","Fragment","Nb.genes","Category","Nb.phage.hallmark.genes","Phage.gene.enrichment.sig","Non-Caudovirales.phage.gene.enrichment.sig","Pfam.depletion.sig","Uncharacterized.enrichment.sig","Strand.switch.depletion.sig","Short.genes.enrichment.sig"))
colnames(vs.pred)<-colnames(vs.head)
colnames(vs.pred)[1]<- "vs.id"
vs.cats <- do.call(rbind,strsplit(x=as.character(vs.pred$vs.id[grep("category",vs.pred$vs.id)]),split=" - ",fixed=T))[,2]
vs.num <- grep("category",vs.pred$vs.id)
vs.pred$Category <- paste(c("",rep.int(vs.cats, c(vs.num[-1],nrow(vs.pred)) - vs.num)), vs.pred$Category)
vs.pred <- vs.pred[-grep("#",vs.pred$vs.id),]
vs.pred$node <- gsub(pattern="VIRSorter_",replacement="",x=vs.pred$vs.id)
vs.pred$node <- gsub(pattern="-circular",replacement="",x=vs.pred$node)
vs.pred$node <- gsub(pattern="cov_(\\d+)_",replacement="cov_\\1.",x=vs.pred$node,perl=F)

VirSorter <- vs.pred %>% select(5,13)
VirSorter$node <- gsub("(^|[^0-9])0+", "\\1", VirSorter$node, perl = TRUE)
VirSorter$Virsorter_prediction <- ifelse(VirSorter$Category == "Prophages 3", "-",
                                         ifelse(VirSorter$Category == "Complete phage contigs 3", "-",
                                                ifelse(VirSorter$Category == "-", "-", "Phage")))
VirSorter <- VirSorter %>% select(2,3)
colnames(VirSorter)<- c("ContigID","VirSorter_prediction")
VirSorter$ContigID <- as.numeric(as.character(VirSorter$ContigID))
VirSorter$ContigID <- sprintf("%010d", VirSorter$ContigID)

# Combine AMR/MGE with Phage prediction
AMR_MGE <- merge(AMR_Plasmid, Phage_prediction, by="ContigID", all = TRUE)
AMR_MGE <- AMR_MGE %>% mutate_if(is.factor, fct_explicit_na, na_level = "-")
AMR_MGE <- AMR_MGE %>% select(2,4,3,5,6,7,10,11)
AMR_MGE$GeneID <- sprintf("%010d", AMR_MGE$GeneID)

write.csv(AMR_MGE, file=snakemake@output[[1]], row.names=FALSE)

AMR_MGE_report <- AMR_MGE
AMR_MGE_report$origin <- ifelse(AMR_MGE_report$Plasmid_prediction == "chromosome" & AMR_MGE_report$VirFinder_prediction == "-" & AMR_MGE_report$VirSorter_prediction =="-", "chromosome",
                                ifelse(AMR_MGE_report$Plasmid_prediction == "plasmid" & AMR_MGE_report$VirFinder_prediction == "-" & AMR_MGE_report$VirSorter_prediction =="-", "plasmid",
                                       ifelse(AMR_MGE_report$Plasmid_prediction == "unclassified" & AMR_MGE_report$VirFinder_prediction == "Phage" | AMR_MGE_report$Plasmid_prediction == "unclassified" & AMR_MGE_report$VirSorter_prediction =="Phage", "phage",
                                              ifelse(AMR_MGE_report$Plasmid_prediction == "unclassified" & AMR_MGE_report$VirFinder_prediction == "-" & AMR_MGE_report$VirSorter_prediction =="-", "unclassified", "ambiguous"))))
AMR_MGE_report <- AMR_MGE_report %>% select(1,2,3,4,5,9)

write.csv(AMR_MGE_report, file=snakemake@output[[2]], row.names=FALSE)


