#!/usr/bin/env R

library(tidyverse)

# load virulence prediction
Virulence_factors <- read.delim(file=snakemake@input[[1]])
Virulence_factors <- Virulence_factors %>% select(1,5,7)
colnames(Virulence_factors) <- c("GeneID","Virulence_factor_prediction","Virulence_confidence_level")
Virulence_factors$GeneID <- as.numeric(as.character(Virulence_factors$GeneID))
Virulence_factors$GeneID <- sprintf("%010d", Virulence_factors$GeneID)

# load toxin prediction
Toxins <- read.delim(file=snakemake@input[[2]])
Toxins <- Toxins %>% select(1,4,5,6)
colnames(Toxins) <- c("GeneID","Toxin_prediction", "SignalP", "Toxin_confidence_level")
Toxins$GeneID <- as.numeric(as.character(Toxins$GeneID))
Toxins$GeneID <- sprintf("%010d", Toxins$GeneID)

# load AMR prediction
AMR_MGE <- read.delim(file=snakemake@input[[3]])
colnames(AMR_MGE) <- c("GeneID","Gene_Name","Contig_Name","ARG_prediction","AMR_Category","MGE_prediction")

# Combine files
Predictions_dfs<-list(Toxins, Virulence_factors, AMR_MGE)
PathoFact_predictions<-Reduce(full_join, Predictions_dfs)
PathoFact_predictions <- PathoFact_predictions[,c(1,7,8,3,2,4,5,6,9,10,11)]
PathoFact_predictions <- PathoFact_predictions %>% filter(GeneID != "        NA")
write.csv(PathoFact_predictions, file=snakemake@output[[1]], row.names=FALSE)


